local M = {}
--[[
    Hello, this is the source code for the More Features Mod, you may look and modify stuff as you wish, 
    but you are not allowed to re-publish the mod or any code that you used in this without my permission.

    StinchinStein
]]--
local sensors = require("sensors")
local drivetrain = require("drivetrain")
local engine = require("engine")
local electrics = require("electrics")
local turbocharger = require("turbocharger")
local beamstate = require("beamstate")
M.SoundEffects = require("SoundEffects")
M.TransmissionSystem = require("TransmissionSystem")
M.EngineSystem = require("EngineSystem")
M.BatterySystem = require("BatterySystem")

local sirenState = 0

local jsonFile = nil
local jsonDefaultFile = nil
M.configurationFile = nil

local oilTempF = 0
local checkSounds = 0

M.crashTimer = 0
M.justCrashed = false
M.justAlarmOff = false
M.justAlarmOffTimer = 0
M.coldStartDelay = 0
M.gForce = nil
M.alarmOn = false
M.alarmTimer = 0
--[[
0 = Entirely Off
1 = On/ACC
]]--

local onLoopTimer = 0
local onLoopAmount = 0

M.brakeWear = 0


local blinkTimerThreshold = 0.4
local blinkTimer = 0

M.vehicleSts = nil
M.lightbarFound = false

local justInitialized = true
local pitchSpeed = 0
local preParkingValue = nil

local vehicleName = ""

local initialized = false
local usingGamepad = false --Button Clutch instead of axis

--ELECTRICS OVERRIDE


local smoothers = {
    wheelspeed = newExponentialSmoothing(10),
    gear_A = newExponentialSmoothing(10),
    gear_M = newExponentialSmoothing(10),
    rpm = newExponentialSmoothing(10),
    lights = newExponentialSmoothing(10),
    fuel = newExponentialSmoothing(10),
    oiltemp = newExponentialSmoothing(10),
    watertemp = newExponentialSmoothing(10),
    turnsignal = newExponentialSmoothing(10),
    airspeed = newExponentialSmoothing(10),
    altitude = newExponentialSmoothing(10)
}

local lightsState = 0

local wheelObj = -1
local wheelRadius = 0

local signalRightState = false
local signalLeftState = false
local signalWarnState = false
local lightbarState = 0

local fogLightsState = false

local blinkPulse = false
local blinkTimerThreshold = 0.4
local blinkTimer = 0
local rpmspin = 0

local lastValues = {}
local smoothValuesExp = {}
M.disabledState = {}


local blinkTest = false
local function generateBlinkPulse(dt)
    blinkTimer = blinkTimer + dt
    if(blinkTimer > blinkTimerThreshold) then
        blinkPulse = not blinkPulse
        blinkTest = not blinkTest 

        if M.configurationFile.blinkerSounds then
            if(((electrics.values.signal_right_input == 1 or electrics.values.signal_left_input == 1) and M.EngineSystem.ignitionState == 1) or (M.EngineSystem.ignitionState == 0 and electrics.values.signal_right_input == 1 and electrics.values.signal_left_input == 1) or M.alarmOn) then
                if(blinkTest) then
                    if(M.alarmOn and M.configurationFile.alarmEnabled) then
                        signalRightState = true
                        signalLeftState = true
                        M.SoundEffects.playPitchSound("HornAlertSound", 1, 1, 1) --speed / 70
                    elseif(M.justAlarmOff) then
                        signalRightState = false
                        signalLeftState = false
                    end
                    M.SoundEffects.playPitchSound("BlinkerOffSound", 1, 1, 1)
                else 
                    M.SoundEffects.playPitchSound("BlinkerOnSound", 1, 1, 1)
                end
            end
        end
        blinkTimer = 0
    end
end

local function alarmOff()
    if(M.alarmOn) then
        M.alarmOn = false
        M.justAlarmOff = true
        signalLeftState = false
        signalRightState = false
        signalWarnState = false
    end
end
-- user input functions
local function toggle_left_signal()
    if not signalWarnState then
        signalLeftState = not signalLeftState
    else
        signalLeftState = true
    end
    if signalLeftState then
        signalRightState = false
        signalWarnState = false
    end
end

local function toggle_right_signal()
    if not signalWarnState then
        signalRightState = not signalRightState
    else
        signalRightState = true
    end
    if signalRightState then
        signalLeftState = false
        signalWarnState = false
    end
end

local function toggle_warn_signal()
    signalWarnState = not signalWarnState
    signalRightState = signalWarnState
    signalLeftState  = signalWarnState
end

local function toggle_lights()
    lightsState = lightsState + 1
    if lightsState == 3 then lightsState = 0 end
end

local function toggle_lightbar_signal()
    lightbarState = lightbarState + 1
    if lightbarState == 2 then lightbarState = 0 end
end

local function setLightsState(newval)
    lightsState = newval
end

local function toggle_fog_lights()
    fogLightsState = not fogLightsState
end



local function updateElectrics(dt)

    local vals = electrics.values
    -- the primary source values
    vals.running = (drivetrain.engineDisabled == false)

    -- steering
    local _, h
    for _, h in pairs (hydros.hydros) do
        if h.inputSource == "steering_input" then
            if h.hydroDirState then
                vals.steering = -h.hydroDirState * v.data.input.steeringWheelLock
                break
            end
        end
    end


    if(M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and M.EngineSystem.ignitionState == 0 and math.abs(drivetrain.avgAV) * wheelRadius < 2) then
        vals.wheelspeed = 0
    else
        if wheelObj ~= nil then
            vals.wheelspeed = math.abs(drivetrain.avgAV) * wheelRadius
        end
    end

    vals.driveshaft = drivetrain.driveshaftAngle * (360 / math.pi)
    vals.axle = drivetrain.axleAngle * (360 / math.pi)

    -- P R N D 2 1
    -- 0 1 2 3 4 5
    if drivetrain.shifterMode == 3 then
        vals.gear_A = drivetrain.shifterPosition + 2
    else
        vals.gear_A = math.min(math.max(drivetrain.gear + 2, 1), 3)
        if vals.parkingbrake_input ~= nil and vals.parkingbrake_input > 0 then vals.gear_A = 0 end
    end
    vals.gear_A = vals.gear_A * (1/5)

    vals.gear_M = drivetrain.gear

    if obj.wheel_count > 0 and (not M.EngineSystem.startingVehicle) or (input.keys.E and not M.EngineSystem.engineIsOff) then
        vals.rpm = drivetrain.rpm
        rpmspin = rpmspin + (dt * drivetrain.rpm)
        if rpmspin > 360 then rpmspin = rpmspin - 360 end
        vals.rpmspin = rpmspin
    end

    vals.brake = drivetrain.brake
    vals.throttle = drivetrain.throttle
    vals.engineThrottle = drivetrain.engineThrottle
    vals.clutch = drivetrain.clutch
    vals.parkingbrake = vals.parkingbrake_input 
    if((M.BatterySystem.batteryDead and M.EngineSystem.engineIsOff) or M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and M.EngineSystem.ignitionState == 0) then
        vals.lights = false
        vals.fuel = 0
        vals.oiltemp = 0
        vals.watertemp = 0
    else
        vals.lights = lightsState
        vals.fuel = drivetrain.fuel / drivetrain.fuelCapacity
        vals.oiltemp = drivetrain.engine.oilTemperature
        vals.watertemp = drivetrain.engine.coolantTemperature
    end
    if signalWarnState then vals.turnsignal = 0
    elseif signalRightState then vals.turnsignal = 1
    elseif signalLeftState then vals.turnsignal = -1
    else vals.turnsignal = 0
    end

    local vel = obj:getVelocity()
    vals.airspeed = vel:length()
    vals.virtualAirspeed = sensors.virtualAirspeed
    vals.verticalVelocity = vel.z
    vals.altitude = obj:getPosition().z
    vals.parking = 0 -- TODO: input.parkinglights
    vals.reverse = (drivetrain.gear < 0)

    -- highbeam

    -- and then the derived values
    if((M.BatterySystem.batteryDead and M.EngineSystem.engineIsOff) or M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and M.EngineSystem.ignitionState == 0) then
        vals.signal_L = (vals.signal_right_input == 1 and vals.signal_left_input == 1 and blinkPulse) or false
        vals.signal_R = (vals.signal_right_input == 1 and vals.signal_left_input == 1 and blinkPulse) or false
        vals.signal_right_input = (signalRightState)
        vals.signal_left_input = (signalLeftState)
        vals.hazard = (signalWarnState and blinkPulse)
        vals.lightbar = false
        vals.checkengine = false
        vals.ignition = false
        vals.lowfuel = false
        vals.lowpressure = false
        vals.oil = false
        vals.abs = false
        vals.esc = false
        vals.tcs = false
        vals.lowhighbeam = false
        vals.lowbeam = false
        vals.highbeam = false
        vals.fog = false
    elseif(M.EngineSystem.ignitionState ~= 0) then
        vals.signal_L = (electrics.values.signal_left_input == 1 and blinkPulse)
        vals.signal_R = (electrics.values.signal_right_input == 1 and blinkPulse)
        vals.signal_right_input = (signalRightState)
        vals.signal_left_input = (signalLeftState)
        vals.hazard = (signalWarnState and blinkPulse)
        vals.lightbar = lightbarState
        vals.checkengine = (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1) or drivetrain.engineDisabled == true
        vals.ignition = (drivetrain.engineDisabled == false)
        vals.lowfuel = (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1 and M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and not M.EngineSystem.startingVehicle and not M.EngineSystem.vehicleAlreadyOn) or (drivetrain.fuel < (0.1 * drivetrain.fuelCapacity))
        vals.lowpressure = (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1 and M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and not M.EngineSystem.vehicleAlreadyOn) or (beamstate.lowpressure)
        vals.oil = (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1 and M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and not M.EngineSystem.startingVehicle and not M.EngineSystem.vehicleAlreadyOn) or drivetrain.engine.oilTemperature >= 130
        vals.abs = (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1 and M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and not M.EngineSystem.startingVehicle and not M.EngineSystem.vehicleAlreadyOn) or drivetrain.absPulse
        vals.esc = (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1 and M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and not M.EngineSystem.startingVehicle and not M.EngineSystem.vehicleAlreadyOn) or drivetrain.esc.escPulse
        vals.tcs = drivetrain.esc.tcsPulse or (M.EngineSystem.engineIsOff and M.EngineSystem.ignitionState == 1 and M.EngineSystem.ignitionTimer < M.EngineSystem.ignitionTimerThreshhold and not M.EngineSystem.vehicleAlreadyOn)
        vals.lowhighbeam = (electrics.values.lights == 1 or electrics.values.lights == 2)
        vals.lowbeam = (electrics.values.lights == 1)
        vals.highbeam = (electrics.values.lights == 2)
        vals.fog = fogLightsState
    end

    for f, v in pairs(vals) do
        if M.disabledState[f] ~= nil then
            vals[f] = nil
        else
            if type(v) == 'boolean' then
                vals[f] = vals[f] and 1 or 0
            end
        end
    end

    for f, s in pairs(smoothers) do
        if vals[f] ~= nil then
            vals[f] = s:get(vals[f])
        end
    end

    for f, v in pairs(vals) do
        if v ~= lastValues[f] then
            material.funcChanged(f, v)
        end
        lastValues[f] = v
    end

    electrics.values = vals
    gui.send("electrics", vals)
end

local function init()
    M.vehicleSts = readJsonFile("game:".. v.vehicles.vehicleDirectory .."info.json") or readJsonFile("lua/vehicle/VehicleSettings/defaultInfo.json")    

    vehicleName = v.main.information.name
    print("Configuration File Name Should Be \"".. vehicleName .. ".json\"")
    jsonFile = readJsonFile("lua/vehicle/VehicleSettings/".. vehicleName ..".json")
    jsonDefaultFile = readJsonFile("lua/vehicle/VehicleSettings/defaultValues.json")

    if(jsonFile == nil) then
        print("unable to read json file, using default values")
        jsonFile = readJsonFile("lua/vehicle/VehicleSettings/defaultValues.json") 
    else
        print("read json file "..jsonFile.vehicleName)
    end

    for k,v in pairs(v.data.groups) do
        if(string.match(k, "lightbar")) then
            M.lightbarFound = true
        end
    end

    if(jsonFile ~= nil) then
        if(jsonFile.vehicleName ~= nil) then jsonDefaultFile.vehicleName = jsonFile.vehicleName end
        if(jsonFile.stallRPM ~= nil) then jsonDefaultFile.stallRPM = jsonFile.stallRPM end
        if(jsonFile.startVehicleTimer ~= nil) then jsonDefaultFile.startVehicleTimer = jsonFile.startVehicleTimer end
        if(jsonFile.canStall ~= nil) then jsonDefaultFile.canStall = jsonFile.canStall end
        if(jsonFile.newClutchSystem ~= nil) then jsonDefaultFile.newClutchSystem = jsonFile.newClutchSystem end
        if(jsonFile.blinkerSounds ~= nil) then jsonDefaultFile.blinkerSounds = jsonFile.blinkerSounds end
        if(jsonFile.parkingBrakeSounds ~= nil) then jsonDefaultFile.parkingBrakeSounds = jsonFile.parkingBrakeSounds end
        if(jsonFile.reverseSoundType ~= nil) then jsonDefaultFile.reverseSoundType = jsonFile.reverseSoundType end
        if(jsonFile.brakeSquealSounds ~= nil) then jsonDefaultFile.brakeSquealSounds = jsonFile.brakeSquealSounds end
        if(jsonFile.compatibleWithMod ~= nil) then jsonDefaultFile.compatibleWithMod = jsonFile.compatibleWithMod end
        if(jsonFile.gaugeStartupType ~= nil) then jsonDefaultFile.gaugeStartupType = jsonFile.gaugeStartupType end
        if(jsonFile.startPitchModifier ~= nil) then jsonDefaultFile.startPitchModifier = jsonFile.startPitchModifier end
        if(jsonFile.hazardCrash ~= nil) then jsonDefaultFile.hazardCrash = jsonFile.hazardCrash end
        if(jsonFile.alarmEnabled ~= nil) then jsonDefaultFile.alarmEnabled = jsonFile.alarmEnabled end
    end
    M.configurationFile = jsonDefaultFile

    if(M.configurationFile.compatibleWithMod and M.vehicleSts.Type == "Truck" or M.vehicleSts.Type == "Car" or M.vehicleSts.Type == "Small Vehicle" or M.vehicleSts.Type == "Aircraft") then
        M.SoundEffects.init()
        M.BatterySystem.init()
        M.TransmissionSystem.init()
        M.EngineSystem.init()

        wheelRadius = 0
        for wi,wd in pairs(drivetrain.wheelInfo) do
            if wd.speedo ~= nil and wd.speedo == true then
                wheelObj = obj:getWheel(wd.wheelID)
                wheelRadius = wd.radius
                break
            end
        end
        oilTempF = math.floor((engine.oilTemperature * 1.8) + 32)
        --M.coldStartDelay = -( (((oilTempF) * 70) - 8000) )

        M.coldStartDelay = 0

        coldStartDelay = oilTempF * 40
        signalRightState = false
        signalLeftState = false
        signalWarnState = false

        M.alarmOn = false
        M.alarmTimer = 0
        justInitialized = true

        if(electrics.values.parkingbrake) then preParkingValue = math.floor(electrics.values.parkingbrake) end
    end
    initialized = true
end


--Blinker click sounds for when it's enabled
local clickSound1 = false
local clickSound2 = false
local clickSound3 = false
local clickedAlready = false
local steerWentL = false
local steerWentR = false
local justTurnedOn = false
local justStartedEngine = false
local revBeepTimer = 0
local hornTimer = 0
local pSTimer = 0
local alarmKey = 0


local function update(dt)
    if(initialized) then
        if((M.vehicleSts.Type == "Truck" or M.vehicleSts.Type == "Car" or M.vehicleSts.Type == "Small Vehicle" or M.vehicleSts.Type == "Aircraft")) then
            
            local skinName = v.data.options.skinName

            if(M.configurationFile.compatibleWithMod) then

                --Transmission System
                if(M.configurationFile.newClutchSystem and drivetrain.shifterMode == 0) then 
                    M.TransmissionSystem.update(dt)
                else
                    M.TransmissionSystem.disable()
                end

                --Battery System
                M.BatterySystem.update(dt)

                --Sound Effects
                M.SoundEffects.update(dt)

                --Engine System
                M.EngineSystem.update(dt)

                if(input.keys.G and alarmKey == 0 and M.configurationFile.alarmEnabled) then
                    if(not M.alarmOn) then
                        M.alarmOn = true
                        gui.message("Alarm Enabled", 5, "disp.alarm")
                    else
                        gui.message("Alarm Disabled", 5, "disp.alarm")
                        alarmOff()
                    end
                    alarmKey = 1
                end
                if(not input.keys.G) then
                    alarmKey = 0
                end

                --Police Siren
                if(lightbarState == 1 and sirenState == 0 and M.lightbarFound) then
                    M.SoundEffects.playPitchSound("PoliceSirenSound", 1, 1, 1)
                    sirenState = 1
                end
                if(lightbarState == 0) then sirenState = 0 end
                if(lightbarState == 1 and M.lightbarFound) then
                    pSTimer = pSTimer + dt
                    if(pSTimer > 4.4) then
                        M.SoundEffects.playPitchSound("PoliceSirenSound", 1, 1, 1)
                        pSTimer = 0
                    end
                end

                M.gForce = math.abs(sensors.gx) + math.abs(sensors.gy)
                if(M.justAlarmOff) then
                    M.justAlarmOffTimer = M.justAlarmOffTimer + dt
                    if(M.justAlarmOffTimer > 1) then
                        M.justAlarmOff = false
                        M.justAlarmOffTimer = 0
                    end
                end
                if(signalWarnState and M.alarmOn) then
                    alarmOff()
                end
                M.brakeWear = M.brakeWear + ((drivetrain.avgAV / 120000) * drivetrain.brake)
                generateBlinkPulse(dt)
                local speed = obj:getVelocity():length() * 2.236 -- speed

                if(M.gForce > 16 and M.EngineSystem.ignitionState == 0 and M.configurationFile.alarmEnabled) then
                    M.alarmOn = true
                end
                M.canStartEngine = true
                local eng = v.data.engine
                local engineRPM = drivetrain.engineAV  * 9.5493

                local steerAngle = math.abs(math.floor(electrics.values.steering))
                local steerAngleNoAbs = (math.floor(electrics.values.steering))
                local maxSteer = math.abs(math.floor(v.data.input.steeringWheelLock))
                
                electrics.toggle_left_signal = toggle_left_signal
                electrics.toggle_right_signal = toggle_right_signal
                electrics.toggle_lights = toggle_lights
                electrics.toggle_fog_lights = toggle_fog_lights
                electrics.toggle_lightbar_signal = toggle_lightbar_signal
                electrics.toggle_warn_signal = toggle_warn_signal
                electrics.update = updateElectrics

                if((electrics.values.signal_right_input == 1 or electrics.values.signal_left_input == 1) and not (electrics.values.signal_left_input == 1 and electrics.values.signal_right_input == 1)) then
                    if(steerAngle > (maxSteer / 2) - (maxSteer / 4) and not clickSound1) then
                        M.SoundEffects.playPitchSound("BlinkerClickSound", 1, 0.7, 1.3) --speed / 70
                        clickSound1 = true
                        clickedAlready = true
                        if(steerAngleNoAbs > 0) then steerWentL = true end
                        if(steerAngleNoAbs < 0) then steerWentR = true end
                    end
                    if(steerAngle > (maxSteer / 2) and not clickSound2) then
                        M.SoundEffects.playPitchSound("BlinkerClickSound", 1, 0.7, 1.3) --speed / 70
                        clickSound2 = true
                        clickedAlready = true
                        if(steerAngleNoAbs > 0) then steerWentL = true end
                        if(steerAngleNoAbs < 0) then steerWentR = true end
                    end
                    if(steerAngle > (maxSteer / 2) + (maxSteer / 4) and not clickSound3) then
                        M.SoundEffects.playPitchSound("BlinkerClickSound", 1, 0.7, 1.3) --speed / 70
                        clickSound3 = true
                        clickedAlready = true
                        if(steerAngleNoAbs > 0) then steerWentL = true end
                        if(steerAngleNoAbs < 0) then steerWentR = true end
                    end

                    if(steerAngle > -50 and steerAngle < 50 and clickedAlready) then
                        clickSound1 = false
                        clickSound2 = false
                        clickSound3 = false
                        if(steerWentL and not steerWentR and electrics.values.signal_left_input == 1) then
                            electrics.toggle_left_signal()
                        elseif(steerWentL and not steerWentR and electrics.values.signal_left_input == 0) then
                            electrics.toggle_right_signal()
                        end
                        if(steerWentR and not steerWentL and electrics.values.signal_right_input == 1) then
                            electrics.toggle_right_signal()
                        elseif(steerWentR and not steerWentL and electrics.values.signal_right_input == 0) then
                            electrics.toggle_left_signal()
                        end
                        steerWentL = false
                        steerWentR = false
                        clickedAlready = false
                    end
                end

                if(M.configurationFile.hazardCrash) then
                    if(M.gForce > 280) then
                        M.alarmOn = true
                    end
                end

                for _, n in ipairs(v.data.engine.waterDamage.nodes) do
                    if obj:inWater(n) then
                        M.canStartEngine = false
                    end
                end
                if(input.keys.H) then
                    hornTimer = hornTimer + dt
                    if(hornTimer > 0.03) then
                        M.SoundEffects.playPitchSound("HornSound", 1, 1, 1)
                        hornTimer = 0
                    end
                end

                checkSounds = checkSounds + dt
                if(checkSounds > 0.2) then

                    --Parking Brake Engage
                    if(M.configurationFile.parkingBrakeSounds) then
                        if(preParkingValue ~= math.floor(electrics.values.parkingbrake) and math.floor(electrics.values.parkingbrake) == 1) then 
                            M.SoundEffects.playPitchSound("ParkingBrakeEngage", 1, 1, 1)
                            preParkingValue = math.floor(electrics.values.parkingbrake) 
                        elseif( math.floor(electrics.values.parkingbrake) ~= 1) then
                            preParkingValue = math.floor(electrics.values.parkingbrake) 
                        end
                    end

                    --Whining
                    if(drivetrain.gear < 0 and not M.EngineSystem.engineIsOff) then 
                        if(speed > 40) then pitchSpeed = 40 else pitchSpeed = speed end

                        if(M.configurationFile.reverseSoundType == "whine" and drivetrain.avgAV < 0) then
                            M.SoundEffects.playPitchSound("GearWhineSound1", 1, speed / 50, 1 + pitchSpeed / 650)
                        elseif(M.configurationFile.reverseSoundType == "beep") then
                            revBeepTimer = revBeepTimer + 1
                            if(revBeepTimer > 4) then
                                SoundEffects.playPitchSound("ReverseBeepSound", 1, 1, 1)
                                revBeepTimer = 0
                            end
                        end
                    end

                    --Brake Squeal
                    if(M.configurationFile.brakeSquealSounds) then
                        if(drivetrain.brake > 0 and M.brakeWear > 25 and (drivetrain.avgAV > 4 or drivetrain.avgAV < -4)) then
                            M.SoundEffects.playPitchSound("BrakeSquealSound", 1, drivetrain.brake / 1.5, 1)
                        end
                    end

                    checkSounds = 0
                end



                if(engine.oilOverheatDamage > (v.data.engine.connectingRodDamageThreshold or 2000000)) then
                    M.canStartEngine = false
                end
            elseif(justInitialized) then
                gui.message("More Features Mod is disabled for ".. vehicleName, 5, "disp.disabledMod")
                justInitialized = false
            end
        elseif(justInitialized) then
            gui.message("More Features Mod is disabled for ".. vehicleName, 5, "disp.disabledMod")
            justInitialized = false
        end
    end
    if(not initialized) then
        init()
    end
end


M.onReset = init
M.updateGFX = update
M.alarmOff = alarmOff
M.updateElectrics = updateElectrics

return M